/*
 * Created on 21.11.2005
 *
 */
package imageintegrity;

import java.io.*;

/**
 * @author Stefan
 *
 */
public class MD5Jpeg implements IMD5Image {
	private String fileName;
	
	public static final int JFIF_SOI = 0xffd8,
	   						JFIF_COMMENT = 0xfffe,
							JFIF_TAG = 0xffe0,
							JFIF_SOS = 0xffda;
	
	
	public MD5Jpeg(String fileName) 
	  throws FileNotFoundException {
		this.fileName = fileName; 
			//new RandomAccessFile(fileName, "r");
	}
	
	
	/* (non-Javadoc)
	 */
	public String getDigest() 
	  throws IOException {
		String comment;
		
		comment = getComment();
		
		if(comment.length() != 32)
			throw new IOException("Comment doesn't contain a valid hash");

		return comment;
	}
	
	public String createDigest(String password) 
		throws IOException {
		RandomAccessFile imgFile = new RandomAccessFile(this.fileName, "r");
		com.twmacinta.util.MD5 md5 = new com.twmacinta.util.MD5();
		int tag, length;
		byte[] buf = new byte[512];

		imgFile.seek(0L);
		
		if((imgFile.readShort() & 0xffff) != JFIF_SOI || (imgFile.readShort() & 0xffff) != JFIF_TAG)
			throw new IOException("Illegal file format ");
		
		length = ((imgFile.readShort() & 0xffff) - 2);
		imgFile.skipBytes(length);
		
		while((tag = (imgFile.readShort() & 0xffff)) != JFIF_SOS) {
			length = ((imgFile.readShort() & 0xffff) - 2);			
			imgFile.skipBytes(length);
		}
		
		md5.Init();
		md5.Update(JFIF_SOS);
		
		while((length=imgFile.read(buf)) != -1)
			md5.Update(buf, length);
		
		md5.Update(password);
		md5.Final();
		
		imgFile.close();
		
		return md5.asHex();
	}
	
	public String getComment()
		  throws IOException {
			RandomAccessFile imgFile = new RandomAccessFile(this.fileName, "r");
			int tag, length;
			byte[] digest = null;

			imgFile.seek(0L);
			
			if((imgFile.readShort() & 0xffff) != JFIF_SOI || (imgFile.readShort() & 0xffff) != JFIF_TAG)
				throw new IOException("Illegal file format ");
			
			length = ((imgFile.readShort() & 0xffff) - 2);
			imgFile.skipBytes(length);
			
			while((tag = (imgFile.readShort() & 0xffff)) != JFIF_SOS) {
				if(tag == JFIF_COMMENT) {
					length = imgFile.readShort()  & 0xffff;

					digest = new byte[length-2];
					imgFile.read(digest);
		
					break;
				}
				
				length = ((imgFile.readShort() & 0xffff) - 2);			
				imgFile.skipBytes(length);
			}

			imgFile.close();

			if(digest == null)
				return null;
			
			return new String(digest);
	}

/*	private String hexDigest(byte[] digest) {
		String digits = "0123456789abcdef";
		StringBuffer result = null;
		
		if(digest.length == 32) {
		   result = new StringBuffer(16);
		   
		   for(int i=0; i<32; i++) {
		      
		   }
		}
		
		return result.toString();
	}
*/	
	/* (non-Javadoc)
	 */
	public void setDigest(String digest, boolean overwrite)
		  throws IOException {
			RandomAccessFile imgFile = new RandomAccessFile(this.fileName, "r");
			DataOutputStream tempFile;
			String path, name, comment = null;
			File tmp, jpg;
			int tag, length, index;
			byte[] buf;			

			index = fileName.lastIndexOf('\\');
			path = fileName.substring(0, index);
			name = fileName.substring(index+1, fileName.length());
			tmp = new File(path + "_temp_" + name);
			
			if(tmp.exists())
				tmp.delete();
			
			tempFile = new DataOutputStream(new FileOutputStream(tmp));
			
			if((imgFile.readShort() & 0xffff) != JFIF_SOI)
				throw new IOException("Illegal file format ");

			tempFile.writeShort(JFIF_SOI);

			comment = this.getComment();

			
			if(comment != null) {
				if(!overwrite) {
					tempFile.close();
					imgFile.close();
					tmp.delete();
					
					throw new IOException("Comment: " + comment);
				}
				
				while((tag = (imgFile.readShort() & 0xffff)) != JFIF_COMMENT) {
					tempFile.writeShort(tag);
					length = (imgFile.readShort() & 0xffff);			
					tempFile.writeShort(length);
					buf = new byte[length-2];
					imgFile.read(buf);
					tempFile.write(buf);
				}
				
				imgFile.skipBytes(comment.length() + 2);
			}
			 else {
			 	tag = imgFile.readShort();
				tempFile.writeShort(tag);
				length = (imgFile.readShort() & 0xffff);			
				tempFile.writeShort(length);
				buf = new byte[length-2];
				imgFile.read(buf);
				tempFile.write(buf);
			 }

			
			tempFile.writeShort(JFIF_COMMENT);
			tempFile.writeShort(34);
			tempFile.write(digest.getBytes());
			
			buf = new byte[512];
			
			while((length=imgFile.read(buf)) != -1)
				tempFile.write(buf, 0, length);
			
			imgFile.close();
			tempFile.close();
			
			jpg = new File(this.fileName);
			jpg.delete();
			tmp.renameTo(jpg);
	}


	public String getFileName() {
		return this.fileName;
	}
}
